<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Drp\NativeEventApp\Auth\ContactDetail;
use Drp\NativeEventApp\Auth\ContactDetails;
use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Contracts\MagicLinkSigner as MagicLinkSignerContract;
use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\Fakes\Delegate;
use Drp\NativeEventApp\Test\Support\Fakes\MagicLinkSigner;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class AuthInfoTest extends GraphQLTestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test */
    public function validates_signature(): void
    {
        $delegate = new Delegate;
        $this->instance(MagicLinkSignerContract::class, new MagicLinkSigner(false));
        AuthInfoFinder::setAuthResolver(function ($authIdentifier) use ($delegate) {
            $this->assertEquals(1, $authIdentifier);

            return $delegate;
        });

        $query = '{
        authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            title,
            minimumAppVersion,
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function get_auth_information_using_auth_identifier(): void
    {
        $this->prepareValidDelegate(new Delegate);

        $query = '{
        authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            title,
            minimumAppVersion,
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_only_returns_the_sms_method_if_no_email_set(): void
    {
        $this->prepareValidDelegate(new class extends Delegate {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([
                    new ContactDetail(AuthMethod::SMS(), '07474747474'),
                ]);
            }
        });

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_only_returns_the_email_method_if_no_phone_number_set(): void
    {
        $this->prepareValidDelegate(new class extends Delegate {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([
                    new ContactDetail(AuthMethod::EMAIL(), 'test@example.com'),
                ]);
            }
        });

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_throws_an_exception_if_no_auth_methods_are_available(): void
    {
        $this->prepareValidDelegate(new class extends Delegate {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([]);
            }
        });

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    private function prepareValidDelegate(Delegate $delegate): Delegate
    {
        $this->instance(MagicLinkSignerContract::class, new MagicLinkSigner);
        AuthInfoFinder::setAuthResolver(function ($authIdentifier) use ($delegate) {
            $this->assertEquals(1, $authIdentifier);

            return $delegate;
        });

        return $delegate;
    }
}
